/*------------------------------------------------------------------------------------------------------------

 Realisiert mit Tiny25
 
 Interner Oszillator mit 8MHz
 
 Layout:
   n.c.          1    8    +5V
   Led B  (PB3)  2     7    PB2  Led A
   Relais (PB4)  3     6    PB1: AIN1 (Komparator; Spannungsteiler der Trafospannung)
   GND           4     5    PB0: AIN0 (n.c.)

 Wenn Klingelknopf gedrckt wird, sinkt die Versorgungsspannung von 18 auf 10V ab.
 ber einen Spannungsteiler geht die Versorgungsspannung auf den neg. Eingang des Komparators.
 Der pos. Eingang liegt auf 1.1V.
 Das Ansteuern des Relais lt die Spannung nicht so weit absinken.

*/

#define F_CPU 8000000UL	/* CPU clock in Hertz */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#define U08 unsigned char
#define U16 unsigned int

volatile U08 gedrueckt;
volatile U16 freigabe;
volatile U16 keinDauerklingeln;




ISR(TIM0_OVF_vect){ //33ms sind rum
static U08 tic;

  if (freigabe) freigabe--;
  if (ACSR & (1<<ACO)){  //Gedrueckt. Ist dann wenn negativer Komparatoreingang unter 1.1V
    if (gedrueckt < 150 ){    //5 Sec. Klingeln bis es richtig laut wird
       gedrueckt++;
    }else{
       freigabe=30*15; //15s freigabe
    }
    if (freigabe && keinDauerklingeln) keinDauerklingeln--;
    PORTB |= 8; //LED Rot
    PORTB &= ~4;//    ein
  }else{
    gedrueckt=0;
    keinDauerklingeln=30*10; //Nicht lnger als 10s am Stck luten (Tastenklemmerschutz)
    PORTB &=~ 8;//LED Grn
    tic++;
    if (tic>30) tic=0;

    if (tic <1 ){ //LED grn blitzen lassen
      PORTB |= 4; //LED ein
    }else{
      PORTB &= ~4;//LED aus
    }
  }
}



int main(void){
  cli();
  
  DDRB = 0x1C; //PB2..4 sind Ausgang
  PORTB = 0x08; //Led rot

  ACSR = (1<<ACBG);  //1.1V Referenz des Komparators
  DIDR0=3; //Digitale Eingnge abschalten

  //Timer 0: 30Hz Zeitbasis
  TCCR0B = (1<<CS02)|(1<<CS00); // 8MHz/1024   (/256 =30,51Hz)
  TIMSK = 1<<TOIE0;

  keinDauerklingeln=0;
  gedrueckt=0;
  freigabe=0;

  sei();

  MCUCR = (1<<SE); //Sleep enable
  asm("sleep");

  while(1) {
    if (freigabe && keinDauerklingeln){
      if (gedrueckt>0){
        PORTB |= 0x10; //Relais ein
      }else{
        PORTB &=~0x10; //Relais aus
      }
    }else{
      PORTB &=~0x10;   //Relais aus
      MCUCR = (1<<SE); //Sleep enable
      asm("sleep");
    }
  }
}
